#include "config.h"
#include <stdio.h>
#include <string.h>
#ifdef BINARYFILEMODE
#include <fcntl.h>  /* for setmode() */
#endif
#include <stdlib.h>
#include <time.h>
#if HAVE_UNISTD_H
# include <sys/types.h>
# include <unistd.h>
#endif
#if HAVE_SYS_PARAM_H
# include <sys/param.h>
#else
#define MAXPATHLEN 256
#endif
#include "common.h"
#include "command.h"
#include "parse.h"
#include "sendcmd.h"
#ifdef X68
#include "tty_x68.h"
#else
#ifdef WIN32
#include "tty_w32.h"
#include "getopt.h"
#else
#ifdef OS2
#include "tty_os2.h"
#else
#ifdef DOS
#include "tty_dos.h"
#else
#include "tty.h"
#include <termios.h>
#endif /* DOS */
#endif /* OS2 */
#endif /* WIN32 */
#endif /* X68 */

extern int	optind, opterr;
extern char	*optarg;

int	verbose = 0;
#ifdef DOS
static unsigned short  speed = 0;
#else
static int     speed = 0;
#endif

static	int	errflg = 0;

#ifndef DONTCAREUID
static	uid_t	uid, euid;
static	gid_t	gid, egid;
static	int	uidswapped = 0;
#endif

void usage()
{
  static	char	*usagestr[] =  {
    "ircom (Ver 0.01) Programmed by Ken-ichi HAYASHI\n",
    "\t -h             : show this usage.\n",
    "\t -r filename    : record IR wave from remocom, and save to file.\n",
    "\t -p filename    : send recorded IR wave file.\n",
    "\t -M makername   : set preset VTR or TV maker as 'makername'.\n",
    "\t -P presetcmd   : send preset IR wave command.\n",
    "\t -w second      : wait(sleep).\n",
    "\t -z             : show informations about IR commander\n",
    "\t -v             : verbose mode.\n",
    "\t -S speed       : serial speed. [normal middle high]\n",
#ifndef X68
    "\t -D ttydevice   : set tty(cua) device.\n",
#endif
    (char *)NULL,
  };
  char	**p;

  p = usagestr;
  while (*p)
    fprintf(stderr, *p++);
}

void Exit(code)
     int code;
{
  if (!(IRgetfd() < 0)){
    IRreset();
    closetty(IRgetfd());
  }
  exit(code);
} 

#ifndef DONTCAREUID
void
daemonuid()
{
  if (uidswapped) {
#ifdef HAVE_SETREUID
    setreuid(uid, euid);
    setregid(gid, egid);
#else
    setuid(uid);
    seteuid(euid);
    setgid(gid);
    setegid(egid);
#endif
    uidswapped = 0;
  }
}

void
useruid()
{
  if (!uidswapped) {
#ifdef HAVE_SETREUID
    setregid(egid, gid);
    setreuid(euid, uid);
#else
    setgid(egid);
    setegid(gid);
    setuid(euid);
    seteuid(uid);
#endif
    uidswapped = 1;
  }
}
#endif


void
main(argc, argv)
     int	argc;
     char	**argv;
{
  char	*devpath = NULL;
  char *def = NULL;
  u_char makercode;
  u_char operationcode;
  int	c;
  int i;
  unsigned char buf[IRRECORDBUF];
  unsigned char u;

#ifndef DONTCAREUID
  uid = getuid();
  euid = geteuid();
  gid = getgid();
  egid = getegid();
  useruid();
#endif

  devpath = getenv("IRCOMTTY");

  if(devpath == NULL){
    devpath = malloc(sizeof(char) * (strlen(RSPORT) +1));
    if(devpath == NULL) {
      fprintf(stderr, "can't malloc\n");
      exit(1);
    }
    strcpy(devpath, RSPORT);
  }

  def = getenv("IRCOMMAKER");
  if(def == NULL)
    makercode = DEFAULT_MAKER;
  else {
    makercode = str_to_makercode(def);
  }

  for(i = 0 ; i < argc; i++){
    if(strcmp("-D", argv[i]) == 0){
      devpath = argv[i+1];
      break;
    }
    if(strcmp("-h", argv[i]) == 0){
      usage();
      exit(-1);
    }
  }

  if (devpath) {
#ifndef DONTCAREUID
    daemonuid();
#endif
    IRsetfd(opentty(devpath));
#ifndef DONTCAREUID
    useruid();
#endif
  }
  if (IRgetfd() < 0)
    Exit(1);

  if(IRcheck() == 0){
    fprintf(stderr, "IR commander is not respond 'OK' message.\n");
    Exit(2);
  }

  IRreset();
  while ((c = getopt(argc, argv, "D:r:p:M:P:S:w:f:vhzZ")) != EOF) {
    switch(c) {
    case 'S':
      switch(optarg[0]){
      case 'h':
      case '3':
	speed = B38400;
	break;
      case 'm':
      case '2':
	speed = B19200;
	break;
      case 'n':
      case '1':
	speed = B9600;
	break;
      default:
	speed = DEFAULTBAUD;
	break;
      }
      IRchangespeed(speed);
      break;
    case 'v':
      verbose = 1;
      break;
    case 'D':
      break; /* do nothing */
    case 'r':
      IRrecordRemocomMode();
      i = IRrecordRemocomData(buf, IRRECORDBUF);
      if((optarg != NULL) && (i > 0)){
	FILE *fp;
	fp = fopen(optarg, WMODE);
	if(fp == NULL){
	  fprintf(stderr, "can't write record file. (%s)\n", optarg);
	  Exit(1);
	}
	fwrite(buf, sizeof(u_char), i, fp);
	fclose(fp);
      }
      break;
    case 'p':
      IRsendRemocomMode();
      sendRecordedData(optarg);
      break;
    case 'w':
      if(optarg != NULL){
	if(verbose)
	  fprintf(stderr, "WAIT %d\n", atoi(optarg));
	sleep(atoi(optarg));
      }
      break;
    case 'M':
      makercode = str_to_makercode(optarg);
      break;
    case 'P':
      IRpresetRemocomMode();
      sendPresetCommand(makercode, optarg);
      break;
    case 'z':
      IRversionInfo();
      break;
    case 'f':
      if(optarg != NULL){
	FILE *fp;
	fp = fopen(optarg, RMODE);
	if(fp == NULL){
	  fprintf(stderr, "can't read sequence file. (%s)\n", optarg);
	  Exit(1);
	}
	parsefile(fp);
	fclose(fp);
      }
      break;
    case 'Z':
      IRrecordRemocomMode();
      i = IRrecordRemocomData(buf, IRRECORDBUF);
      fprintf(stderr, "wait\n");
      sleep(5);
      fprintf(stderr, "send \n");
      IRsendRemocomMode();
      IRsendRemocomData(buf, i);
      break;
    default:
      usage();
      Exit(-1);
    }
    usleep(WAITTIME); /* ?? */
  }

  Exit (errflg ? 1 : 0);
}


